;#
;# lx200h - LX200 compatible telescope control software for the PIC16F870 uC
;# Copyright (C) 2004  Hanno Rein (http://hanno-rein.de) 
;# 
;# This program is free software; you can redistribute it and/or
;# modify it under the terms of the GNU General Public License
;# as published by the Free Software Foundation; either version 2
;# of the License, or (at your option) any later version.
;# 
;# This program is distributed in the hope that it will be useful,
;# but WITHOUT ANY WARRANTY; without even the implied warranty of
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;# GNU General Public License for more details.
;# 
;# You should have received a copy of the GNU General Public License
;# along with this program; if not, write to the Free Software
;# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;# 
;#


    __CONFIG 		_PWRTE_ON & _WDT_OFF & _HS_OSC & _LVP_OFF
    list     		p=16f870
    #include 		<P16f870.INC>
	#include		<hmacro.inc>
	#include		<lcd.inc>
    ERRORLEVEL      -207 



; #############################################
; #####   Declarations                    #####
; #############################################

	focuser_in	equ		5
	focuser_out	equ		4
	ra_w_s		equ		1
	ra_w_f		equ		3
	ra_e_s		equ		2
	ra_e_f		equ		4

	test1 		equ 	0x20
	STATUS_TEMP	equ		0x21
	W_TEMP		equ		0x22
	cmd_c		equ		0x23
	cmd_1		equ		0x24
	cmd_2		equ		0x25
	cmd_3		equ		0x26
	cmd_4		equ		0x27
	cmd_5		equ		0x2F
	cmd_6		equ		0x48
	cmd_7		equ		0x49
	cmd_8		equ		0x4A
	cmd_9		equ		0x4B
	cmd_10		equ		0x4C
	cmd_11		equ		0x4D
	cmd_12		equ		0x4E

	wait10msc	equ		0x29
	wait400usc	equ		0x2A
	loops		equ		0x2B
	loops2		equ		0x2C
	LcdStatus	equ		0x2D
	LcdDaten	equ		0x2E

	a_ra_0		equ		0x30
	a_ra_1		equ		0x31
	a_ra_2		equ		0x32
	a_ra_3		equ		0x58

	a_dec_0		equ		0x33
	a_dec_1		equ		0x34
	a_dec_2		equ		0x35
	a_dec_3		equ		0x57

	digit1		equ		0x36
	digit10		equ		0x37
	digit100	equ		0x38
	sign		equ		0x39
	
	calctemp	equ		0x3A

	config		equ		0x3B
	longformat	equ		0
	sign_dec	equ		1
	sign_tdec	equ		2
	speed		equ		3

	a_time_0	equ		0x3C
	a_time_1	equ		0x3D
	a_time_2	equ		0x3E

	a_date_0	equ		0x3F
	a_date_1	equ		0x40
	a_date_2	equ		0x41

	a_tra_0		equ		0x42
	a_tra_1		equ		0x43
	a_tra_2		equ		0x44

	a_tdec_0	equ		0x45
	a_tdec_1	equ		0x46
	a_tdec_2	equ		0x47
	mouseXPos	equ		0x51
	mouseYPos	equ		0x52
	mouseXPos_TEMP	equ		0x59
	mouseYPos_TEMP	equ		0x5A

	mouseCalc	equ		0x53
	PCLATH_TEMP equ		0x54
	RX_TEMP		equ		0x55
	TMR1_TEMP	equ		0x56

	LcdE		equ		5
	LcdRw		equ		5
	LcdRs		equ		4
	LcdPortE	equ		PORTB
	LcdPort		equ		PORTA
	

; #############################################
; #####   One step equals...              #####
; #############################################

	ratic2		equ		D'4'	;               --> = 4.0000 s
	ratic3		equ		D'61'	; luft bis 256 --> = 0.2383 s
								;                   + --------
								;					  4.2383 s

	dectic2		equ		D'31'	;				--> = 31.000 ' mal 2
	dectic3		equ		D'42' 	; luft bis 256 --> = 00.332 '   (*2!!)
								;                   + --------
								;					  62.332 '
	

	org 0
	goto init


; #############################################
; #####   Interrupt Service Routine       #####
; #############################################
	org  4                  
	MOVWF   W_TEMP			; Backup everything...
    SWAPF   STATUS,W
    CLRF    STATUS
    MOVWF   STATUS_TEMP
    MOVF    PCLATH, W
    MOVWF   PCLATH_TEMP
    CLRF    PCLATH        

	btfsc	PORTC,2			; Axes X or Y?
	goto	mouseXmov
	btfsc	PORTC,3			; Direction
	incf	mouseYPos,f
	btfss	PORTC,3			; Direction
	decf	mouseYPos,f
	goto nomousemov
	mouseXmov:
	btfss	PORTC,3			; Direction
	incf	mouseXPos,f
	btfsc	PORTC,3			; Direction
	decf	mouseXPos,f
	nomousemov:

	bcf    INTCON, INTF     ; Delete RB0-Interrupt Flag 
 	MOVF    PCLATH_TEMP, W	; Restore everything...
    MOVWF   PCLATH
    SWAPF   STATUS_TEMP,W
    MOVWF   STATUS
    SWAPF   W_TEMP,F
    SWAPF   W_TEMP,W 
	retfie


; #############################################
; #####   RS232in                         #####
; #############################################
RS232in:

	MOVFW	RCREG			; Read out first (to enable further receipt)
	movwf	RX_TEMP
	bcf     PIR1,RCIF    

	movlw	0x06			; ACK Request
	subwf	RX_TEMP,0
	BNZ		noack
	out		'P'				; ACK Response
	goto 	rsintEnde

	noack:
	movlw	'#'				; ignore init #
	subwf	RX_TEMP,0
	BNZ		nosquare
	TSTF	cmd_c
	BZ		rsintEnde		; if end of command --> response
	goto	response			


	nosquare:
	movlw	':'				; command start sign ':'?
	subwf	RX_TEMP,0			
	BNZ		nostart
	TSTF	cmd_c			; is it start sign or 00:00:00?
	BNZ		nostart
	incf	cmd_c,f
	goto	rsintEnde

	nostart:
	movlw	D'1'			; Majorcommand
	subwf	cmd_c,w
	BNZ		nomajorcmd
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_1
	goto	rsintEnde

	nomajorcmd:
	movlw	D'2'			; Subcommand
	subwf	cmd_c,w
	BNZ		nosubcommand
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_2
	goto	rsintEnde

	
	nosubcommand:
	movlw	D'3'			; SubSubcommand
	subwf	cmd_c,w
	BNZ		nosub4
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_3
	goto	rsintEnde

	nosub4:
	movlw	D'4'		
	subwf	cmd_c,w
	BNZ		nosub5
   	movlw	'+'
	subwf	RX_TEMP,0
	BZ		rsintEnde
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_4
	goto	rsintEnde

	nosub5:
	movlw	D'5'
	subwf	cmd_c,w
	BNZ		nosub6
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_5
	goto	rsintEnde

	nosub6:
	movlw	D'6'		
	subwf	cmd_c,w
	BNZ		nosub7
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_6
	goto	rsintEnde

	nosub7:
	movlw	D'7'		
	subwf	cmd_c,w
	BNZ		nosub8
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_7
	goto	rsintEnde

	nosub8:
	movlw	D'8'		
	subwf	cmd_c,w
	BNZ		nosub9
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_8
	goto	rsintEnde

	nosub9:
	movlw	D'9'		
	subwf	cmd_c,w
	BNZ		nosub10
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_9
	goto	rsintEnde

	nosub10:
	movlw	D'10'		
	subwf	cmd_c,w
	BNZ		nosub11
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_10
	goto	rsintEnde

	nosub11:
	movlw	D'11'		
	subwf	cmd_c,w
	BNZ		nosub12
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_11
	goto	rsintEnde

	nosub12:
	movlw	D'12'		
	subwf	cmd_c,w
	BNZ		nosub13
	incf	cmd_c,f
	movfw	RX_TEMP
	movwf	cmd_12
	goto	rsintEnde

	nosub13:
	movlw	D'13'		
	subwf	cmd_c,0
	BNZ		rsintEnde
	clrf	cmd_c

	goto rsintEnde	


; #############################################
; #####   Response                        #####
; #############################################
	response:


; 		#############################################
; 		#####   Response  :D#                   #####
; 		#############################################
		movlw	'D'
		subwf	cmd_1,0
		BNZ		noD
		out 	'|'
		out 	'#'
		out 	'#'
		GOTO	RS232inEnd
		noD:

; 		#############################################
; 		#####   Response  :Gxx#                 #####
; 		#############################################
		movlw	'G'
		subwf	cmd_1,0
		BNZ		noG


		movlw	'V'			; seems to be a firmware version request
		subwf	cmd_2,0
		BNZ		noGVF
		movlw	'F'
		subwf	cmd_3,0
		BNZ		noGVF
		out		'1'
		out		'0'
		out		'0'
		noGVF:
		
		movlw	'G'
		subwf	cmd_2,0
		BNZ		noGG
		call	response1
		noGG:
		
		movlw	'P'
		subwf	cmd_2,0
		BNZ		noGP
		call	response4
		noGP:
	
		movlw	'M'
		subwf	cmd_2,0
		BNZ		noGM
		call	response4
		noGM:
	
		movlw	'N'
		subwf	cmd_2,0
		BNZ		noGN
		call	response4
		noGN:
	
		movlw	'O'
		subwf	cmd_2,0
		BNZ		noGO
		call	response4
		noGO:
		
		movlw	'q'
		subwf	cmd_2,0
		BNZ		noGq
		out		'G'
		out		'D'
		out		'#'
		noGq:
		
		movlw	'V'
		subwf	cmd_2,0
		BNZ		noGVP
		movlw	'P'
		subwf	cmd_3,0
		BNZ		noGVP
		call 	response2
		noGVP:
	
		movlw	'd'
		subwf	cmd_2,0
		BNZ		noGd
		call	response3
		noGd:
	
		movlw	'R'					; output RA
		subwf	cmd_2,0
		BNZ		noGR
			movfw	a_ra_0
			Call	RS23210out
			out		':'
			movfw	a_ra_1
			Call	RS23210out
			out		':'
			movfw	a_ra_2
			Call	RS23210out
			out		'#'
		  	GOTO	RS232inEnd
		noGR:
	
		movlw	'D'					; output Declination
		subwf	cmd_2,0
		BNZ		noGD
			movlw	'+'
			BTFSC	config,sign_dec		
			movlw	'-'
			call	RS232out
			movfw	a_dec_0
			Call	RS23210out
			out		.223
			movfw	a_dec_1
			Call	RS23210out
			out		':'
			movfw	a_dec_2
			Call	RS23210out
			out		'#'
			GOTO	RS232inEnd
		noGD:
	
		movlw	'Z'
		subwf	cmd_2,0
		BNZ		noGZ
		call	response3
		noGZ:
	
		movlw	'A'
		subwf	cmd_2,0
		BNZ		noGA
		call	response3
		noGA:
	
		movlw	'r'
		subwf	cmd_2,0
		BNZ		noGr
		call	response5
		noGr:
	
		movlw	'S'
		subwf	cmd_2,0
		BNZ		noGS
		call	response5
		noGS:
	
		movlw	'L'					; output local time
		subwf	cmd_2,0
		BNZ		noGL
			movfw	a_time_0
			Call	RS23210out
			out		':'
			movfw	a_time_1
			Call	RS23210out
			out		':'
			movfw	a_time_2
			Call	RS23210out
			out		'#'
			GOTO	RS232inEnd
		noGL:
	
		movlw	't'
		subwf	cmd_2,0
		BNZ		noGt
		call	response6
		noGt:
	
		movlw	'g'
		subwf	cmd_2,0
		BNZ		noGg
		call	response7
		noGg:
	
		movlw	'C'					; outputu date
		subwf	cmd_2,0	
		BNZ		noGC
			movfw	a_date_0
			Call	RS23210out
			out		'/'
			movfw	a_date_1
			Call	RS23210out
			out		'/'
			movfw	a_date_2
			Call	RS23210out
			out		'#'
			GOTO	RS232inEnd
		noGC:
	
		GOTO	RS232inEnd
		noG:
	
; 		#############################################
; 		#####   Response  :Qxx#                 #####
; 		#############################################
		movlw	'Q'
		subwf	cmd_1,0
		BNZ		noQ


		movlw	'w'					;stop west slewing
		subwf	cmd_2,0
		BNZ		noQw
		bcf		PORTB,ra_w_s
		bcf		PORTB,ra_w_f
		noQw:
	
		movlw	'e'					;stop east slewing
		subwf	cmd_2,0
		BNZ		noQe
		bcf		PORTB,ra_e_s
		bcf		PORTB,ra_e_f
		noQe:
	
		movlw	D'2'				;stop all
		subwf	cmd_c,0
		BNZ		noQQQ
		bcf		PORTB,ra_e_s
		bcf		PORTB,ra_e_f
		bcf		PORTB,ra_w_s
		bcf		PORTB,ra_w_f
		noQQQ:

		GOTO	RS232inEnd
		noQ:


; 		#############################################
; 		#####   Response  :Sxx#                 #####
; 		#############################################
		movlw	'S'
		subwf	cmd_1,0
		BNZ		noS

	
		movlw	'L'						; set Time
		subwf	cmd_2,0
		BNZ		noSL
			movfw	cmd_4
			movwf	digit10
			movfw	cmd_5
			movwf	digit1
			Call	DEC2BIN
			movwf	a_time_0
			movfw	cmd_7
			movwf	digit10
			movfw	cmd_8
			movwf	digit1
			Call	DEC2BIN
			movwf	a_time_1
			movfw	cmd_10
			movwf	digit10
			movfw	cmd_11
			movwf	digit1
			Call	DEC2BIN
			movwf	a_time_2
		noSL:
	
		movlw	'C'						; set Date
		subwf	cmd_2,0
		BNZ		noSC
			movfw	cmd_4
			movwf	digit10
			movfw	cmd_5
			movwf	digit1
			Call	DEC2BIN
			movwf	a_date_0
			movfw	cmd_7
			movwf	digit10
			movfw	cmd_8
			movwf	digit1
			Call	DEC2BIN
			movwf	a_date_1
			movfw	cmd_10
			movwf	digit10
			movfw	cmd_11
			movwf	digit1
			Call	DEC2BIN
			movwf	a_date_2
			out '1'
			out '#'
			out '#'
			GOTO	RS232inEnd
		noSC:
		
		movlw	'r'						; set target RA
		subwf	cmd_2,0
		BNZ		noSr
			movfw	cmd_4
			movwf	digit10
			movfw	cmd_5
			movwf	digit1
			Call	DEC2BIN
			movwf	a_tra_0
			movfw	cmd_7
			movwf	digit10
			movfw	cmd_8
			movwf	digit1
			Call	DEC2BIN
			movwf	a_tra_1
			movfw	cmd_10
			movwf	digit10
			movfw	cmd_11
			movwf	digit1
			Call	DEC2BIN
			movwf	a_tra_2
			Call	UpdateLcdPost
		noSr:
	
		movlw	'd'						; set target DEC
		subwf	cmd_2,0
		BNZ		noSd
			bcf		config, sign_tdec
			movlw	'-'
			subwf	cmd_4,w
			SKPZ
			goto 	keingetz			; shift everything if '-' sign is in command
				bsf		config, sign_tdec
				movfw	cmd_5
				movwf	cmd_4
				movfw	cmd_6
				movwf	cmd_5
				movfw	cmd_8
				movwf	cmd_7
				movfw	cmd_9
				movwf	cmd_8
				movfw	cmd_11
				movwf	cmd_10
				movfw	cmd_12
				movwf	cmd_11
			keingetz:
			movfw	cmd_4
			movwf	digit10
			movfw	cmd_5
			movwf	digit1
			Call	DEC2BIN
			movwf	a_tdec_0
			movfw	cmd_7
			movwf	digit10
			movfw	cmd_8
			movwf	digit1
			Call	DEC2BIN
			movwf	a_tdec_1
			movfw	cmd_10
			movwf	digit10
			movfw	cmd_11
			movwf	digit1
			Call	DEC2BIN
			movwf	a_tdec_2
			Call	UpdateLcdPost
		noSd:
		
		out 	'1'
		GOTO	RS232inEnd

		noS:


; 		#############################################
; 		#####   Response  :Lxx#                 #####
; 		#############################################
		movlw	'L'
		subwf	cmd_1,0
		BNZ		noL


		movlw	'o'
		subwf	cmd_2,0
		BNZ		noLo
		out 	'1'
		noLo:
	
		movlw	's'
		subwf	cmd_2,0
		BNZ		noLs
		out 	'1'
		noLs:
		
		movlw	'I'
		subwf	cmd_2,0
		BNZ		noLI
		call	response2
		noLI:
	
		movlw	'f'
		subwf	cmd_2,0
		BNZ		noLf
		call	response2
		noLf:
	
		GOTO	RS232inEnd
		noL:


; 		#############################################
; 		#####   Response  :Fxx#                 #####
; 		#############################################
		movlw	'F'
		subwf	cmd_1,0
		BNZ		noF


		movlw	'Q'				; stop focuser movement
		subwf	cmd_2,0
		BNZ		noFQ
		bcf		PORTC,focuser_in
		bcf		PORTC,focuser_out
		noFQ:
	
		movlw	'-'				; focus -
		subwf	cmd_2,0
		BNZ		noFminus
		bsf		PORTC,focuser_in
		noFminus:
	
		movlw	'+'				; focus +
		subwf	cmd_2,0
		BNZ		noFplus
		bsf		PORTC,focuser_out
		noFplus:
	
		GOTO	RS232inEnd
		noF:


; 		#############################################
; 		#####   Response  :Rxx#                 #####
; 		#############################################
		movlw	'R'
		subwf	cmd_1,0
		BNZ		noR


		movlw	'G'				; set guide speed
		subwf	cmd_2,0
		BNZ		noRG
		bsf		config,speed
		noRG:
	
		movlw	'C'				; set center speed
		subwf	cmd_2,0
		BNZ		noRC
		bsf		config,speed
		noRC:
	
		movlw	'M'				; set find speed
		subwf	cmd_2,0
		BNZ		noRM
		bcf		config,speed
		noRM:
	
		movlw	'S'				; set slew speed
		subwf	cmd_2,0
		BNZ		noRS
		bcf		config,speed
		noRS:
	
		GOTO	RS232inEnd
		noR:


; 		#############################################
; 		#####   Response  :Mxx#                 #####
; 		#############################################
		movlw	'M'
		subwf	cmd_1,0
		BNZ		noM

	
		movlw	'w'					; west slewing
		subwf	cmd_2,0
		BNZ		noMw
		btfsc	config,speed
		bsf		PORTB,ra_w_f
		btfss	config,speed
		bsf		PORTB,ra_w_s
		noMw:
	
		movlw	'e'					; east slewing
		subwf	cmd_2,0
		BNZ		noMe
		btfsc	config,speed
		bsf		PORTB,ra_e_f
		btfss	config,speed
		bsf		PORTB,ra_e_s
		noMe:
		
		movlw	'S'					; slew to target
		subwf	cmd_2,0
		BNZ		noMS
		out		'0'					; output "possible", but do nothing
		noMS:
	
		GOTO	RS232inEnd
		noM:


; 		#############################################
; 		#####   Response  :Cxx#                 #####
; 		#############################################
		movlw	'C'
		subwf	cmd_1,0
		BNZ		noC


		movlw	'M'					; Telescop = Target 
		subwf	cmd_2,0
		BNZ		noCM
			movfw	a_tra_0
			movwf	a_ra_0
			movfw	a_tra_1
			movwf	a_ra_1
			movfw	a_tra_2
			movwf	a_ra_2
			movfw	a_tdec_0
			movwf	a_dec_0
			movfw	a_tdec_1
			movwf	a_dec_1
			movfw	a_tdec_2
			movwf	a_dec_2
			bcf		config,sign_dec
			btfsc	config,sign_tdec
			bsf		config,sign_dec
			call	UpdateLcdPos
			call 	response2
		noCM:
	
		GOTO	RS232inEnd
		noC:


  	RS232inEnd:
	clrf	cmd_c					; set commandcounter =0
	rsintEnde:
  	
	btfss	RCSTA,	OERR			; check if recieve error occured
	return
	bcf		RCSTA,	CREN			; if so, restart USART interface
	nop
	bsf		RCSTA,	CREN
	clrf	cmd_c					; and set back commandcounter

	return



; #############################################
; #####   Init                            #####
; #############################################
	init:

; init output pins
	BSF     STATUS,RP0         	; Bank 1
	clrf	TRISB				; output RA und lcd
	bsf		TRISB,0
	clrf	TRISA				; output lcd	
	movlw	B'10001100'
	movwf	TRISC
	movlw	0x06				; disable ADC
	movwf	ADCON1
    BCF     STATUS,RP0         	; Bank 0

; init USART
    BSF      STATUS,RP0        ; Bank 1
    MOVLW    0x20              ;
    MOVWF    TXSTA             ;
    MOVLW    D'31'             ; = 9,6 kbps at 20 MHz 
    MOVWF    SPBRG
    BCF      TXSTA, BRGH       ; 
    BCF      STATUS,RP0        ; Bank 0
    MOVLW    0x90              ; reciever RS232
    MOVWF    RCSTA             ; 

; init interrupt
    BSF     STATUS,RP0         ; Bank1
  	BcF     PIE1,RCIE          ; disable receive interrupt
    BCF     PIE1,TMR1IE        ; Disable TMR1 interrupt
	bsf    	OPTION_REG, INTEDG ; 0-1 interrupt at pin RB0 
    BCF     STATUS,RP0         ; Bank 0
    clrf    PIR1               ; delete all interruptflags
    clrf    PIR2
    BSF     INTCON,GIE         ; allow interrupts 
	bsf    	INTCON, INTE       ; allow RB0-interrupt 
    BcF     INTCON,PEIE        ; prohibit PE interrupts

; init timer1
	clrf	TMR1L
	clrf	TMR1H
	movlw	B'00111011'
	movwf	T1CON

; init variables
	clrf	cmd_c
	clrf	a_ra_0
	clrf	a_ra_1
	clrf	a_ra_2
	clrf	a_dec_0
	clrf	a_dec_1
	clrf	a_dec_2
	clrf	a_tra_0
	clrf	a_tra_1
	clrf	a_tra_2
	clrf	a_tdec_0
	clrf	a_tdec_1
	clrf	a_tdec_2
	clrf	config
	clrf	mouseXPos
	clrf	mouseYPos
	movlw	.1
	movwf	a_date_0
	movwf	a_date_1
	movwf	a_date_2
	clrf	a_time_0
	clrf	a_time_1
	clrf	a_time_2


; #############################################
; #####   Init LCD Display                #####
; #############################################
	movlw	D'250'		; 250 ms Pause 
	movwf	loops	
	call	WAIT		
	movlw	B'00000011'	; 1
	movwf	LcdPort
	bsf		LcdPortE, LcdE
	nop	
;	nop
	bcf		LcdPortE, LcdE
	movlw	D'50'		; 50 ms Pause
	movwf	loops
	call	WAIT
	movlw	B'00000011'	; 2
	call	Control8Bit
	movlw	B'00000011'	; 3
	call 	Control8Bit
	movlw	B'00000010'	; 4
	call 	Control8Bit	
	lcdcontrol	B'00000001'	; delete and cusor home
	lcdcontrol	B'00101000'	; 5 function set, 4-bit  2-rows,  5x11
	lcdcontrol	B'00001000'	; 6 display off
	lcdcontrol	B'00000110'	; 7 entry mode, increment, disable display-shift
	lcdcontrol	B'00000011'	; 8 cursor home, cursor home
	lcdcontrol	B'00001100'	; 9 display on, no curser, no block

	call	UpdateLcdPos
	call	UpdateLcdPost
	call	UpdateLcdTime

	lcdcontrol	B'11100010'	; home row2
	lcd	'H'
	lcd	'.'
	lcd	'R'
	lcd	'e'
	lcd	'i'
	lcd	'n'


; #############################################
; #####   MAIN LOOP                       #####
; #############################################
loop:

	btfsc   PIR1,RCIF       ; rs232 data arrival?
    call    RS232in			; get it...

	
	movlw	.16				; timer constant for 1 second
	subwf	TMR1H,w
	SKPZ	
	goto timeweiter1


; 	#############################################
; 	#####   Update Time                     #####
; 	#############################################
		clrf	TMR1H
		clrf	TMR1_TEMP
		incf	a_time_2,f
		movlw	.60
		subwf	a_time_2,w
		SKPZ
		goto	nochangeanymore
		incf	a_time_1,f
		clrf	a_time_2
		movlw	.60
		subwf	a_time_1,w
		SKPZ
		GOTO	nochangeanymore
		incf	a_time_0,f
		clrf	a_time_1
		movlw	.24
		subwf	a_time_0,w
		SKPZ
		GOTO	nochangeanymore
		clrf	a_time_0
		nochangeanymore:
		call	UpdateLcdTime	
		goto 	ra_update
	
	timeweiter1:
	movlw	.8						; do it every second (but 1/2 second after the time update)
	subwf	TMR1H,w
	SKPZ	
	goto 	loop
	movlw	.1	
	subwf	TMR1_TEMP,w
	SKPNZ
	goto 	loop


; 	#############################################
; 	#####   Update RA                       #####
; 	#############################################
		incf	TMR1_TEMP,f
		incf	a_ra_2,f
		movlw	.60
		subwf	a_ra_2,w
		SKPZ
		goto	nochangeanymore2
		incf	a_ra_1,f
		clrf	a_ra_2
		movlw	.60
		subwf	a_ra_1,w
		SKPZ
		GOTO	nochangeanymore2
		incf	a_ra_0,f
		clrf	a_ra_1
		movlw	.24
		subwf	a_ra_0,w
		SKPZ
		GOTO	nochangeanymore2
		clrf	a_ra_0
		nochangeanymore2:
		call	UpdateLcdPos	
 


; 	#############################################
; 	#####   Update RA with encoder data     #####
; 	#############################################
		ra_update:
		TSTF	mouseXPos
		SKPNZ
		goto 	noraupdate
		movfw	mouseXPos
		movwf	mouseXPos_TEMP	
		clrf	mouseXPos
		btfsc	mouseXPos_TEMP,7	
		goto	radecloop
		raincloop:
		call	incRAtic
		decfsz	mouseXPos_TEMP
		goto 	raincloop
		goto 	noraupdate
		radecloop:
		call	decRAtic
		incfsz	mouseXPos_TEMP
		goto 	radecloop
		noraupdate:


; 	#############################################
; 	#####   Update DEC with encoder data    #####
; 	#############################################
		TSTF	mouseYPos
		SKPNZ
		goto 	nodecupdate
		movfw	mouseYPos
		movwf	mouseYPos_TEMP	
		clrf	mouseYPos
		btfsc	mouseYPos_TEMP,7	
		goto	initdecdecloop
		goto	initdecincloop
	
		initdecdecloop:
		NEGF	mouseYPos_TEMP,f
		movfw	mouseYPos_TEMP
		addwf	mouseYPos_TEMP,f	; do it 2 times (easier to calculate, cause dec tic > 1' )
		goto 	decdecloop			; +
	
		initdecincloop:
		movfw	mouseYPos_TEMP
		addwf	mouseYPos_TEMP,f	; do it 2 times (easier to calculate, cause dec tic > 1' )
		goto 	decincloop			; +	
		
		decincloop:
		btfsc	config,sign			; check sign first (bug inside: may count 1 additional tic around 0000'00'' )
		call	decDECtic
		btfss	config,sign			; check sign first
		call	incDECtic
		decfsz	mouseYPos_TEMP
		goto 	decincloop
		goto 	nodecupdate
		decdecloop:
		btfss	config,sign			; check sign first
		call	decDECtic
		btfsc	config,sign			; check sign first
		call	incDECtic
		decfsz	mouseYPos_TEMP
		goto 	decdecloop
		nodecupdate:


	goto loop




; #############################################
; #####   Encoder calculations            #####
; #############################################
incDECtic
	movlw	dectic2
	addwf	a_dec_2,f
	movlw	.60
	subwf	a_dec_2,w
	btfss	STATUS,C	; Carry
	return				; C=0
	movwf	a_dec_2		; C=1
	incf	a_dec_1,f
	movlw	.60
	subwf	a_dec_1,w
	SKPZ
	return
	clrf	a_dec_1		
	incf	a_dec_0,f
	return

decDECtic
	movlw	dectic2
	subwf	a_dec_2,f
	btfsc	STATUS,C	; Carry?
	return
	movlw	.60
	addwf	a_dec_2,f
	decf	a_dec_1,f
	incf	a_dec_1,w	; just to test zero
	SKPZ
	return
	movlw	.60
	addwf	a_dec_1,f	; a_dec_1 = 59
	decf	a_dec_0,f
	incf	a_dec_0,w	; test zero
	SKPZ
	return				; all ok... return
	; change sign:
	clrf	a_dec_0
	clrf	a_dec_1
	NEGF	a_dec_2,f
	movlw	.60
	addwf	a_dec_2,f
	BTFSC	config,sign_dec		; -?
	goto 	minus2plus1
	bsf		config,sign_dec		
	return
	minus2plus1:
	bcf		config,sign_dec		
	return

incRAtic
	movlw	ratic3
	addwf	a_ra_3,f
	ADDCF	a_ra_2,f
	movlw	ratic2
	addwf	a_ra_2,f
	movlw	.60
	subwf	a_ra_2,w
	btfss	STATUS,C
	return				; C=0
	movwf	a_ra_2		; C=1
	incf	a_ra_1,f
	movlw	.60
	subwf	a_ra_1,w
	SKPZ
	return
	incf	a_ra_0,f
	clrf	a_ra_1
	movlw	.24
	subwf	a_ra_0,w
	SKPNZ
	clrf	a_ra_0
	return

decRAtic
	movlw	ratic3
	subwf	a_ra_3,f
	btfss	STATUS,C
	decf	a_ra_2,f
	incf	a_ra_2,w	; test zero
	SKPZ				
	goto 	decRAallesok
	movlw	.60			
	addwf	a_ra_2,f
	decf	a_ra_1,f
	incf	a_ra_1,w	; test zero
	SKPZ				
	goto 	decRAallesok
	movlw	.60
	addwf	a_ra_1,f	; a_ra_1 = 59
	decf	a_ra_0,f
	incf	a_ra_0,w	; test zero
	SKPZ
	goto 	decRAallesok
	movlw	.24
	addwf	a_ra_0,f	; a_ra_0 = 23
	decRAallesok:
	movlw	ratic2
	subwf	a_ra_2,f
	btfsc	STATUS,C
	return				; C=0
	movlw	.60			; C=1
	addwf	a_ra_2,f
	decf	a_ra_1,f
	incf	a_ra_1,w	; test zero
	SKPZ				
	return				; all ok... return
	movlw	.60
	addwf	a_ra_1,f	; a_ra_1 = 59
	decf	a_ra_0,f
	incf	a_ra_0,w	; test zero
	SKPZ
	return				; all ok... return
	movlw	.24
	addwf	a_ra_0,f	; a_ra_0 = 23
	return

	
; #############################################
; #####   Update time on lcd              #####
; #############################################
UpdateLcdTime
	lcdcontrol	B'10010110'		;home row1
	movfw	a_time_0
	Call	lcd10out
	lcd		':'
	movfw	a_time_1
	Call	lcd10out
	lcd		':'
	movfw	a_time_2
	Call	lcd10out
	lcdcontrol	B'11010110'		; home row2
	movfw	a_date_0
	Call	lcd10out
	lcd		'/'
	movfw	a_date_1
	Call	lcd10out
	lcd		'/'
	movfw	a_date_2
	Call	lcd10out
	return


; #############################################
; #####   Update telescope pos on lcd     #####
; #############################################
UpdateLcdPos
	lcdcontrol	B'10000000'		;home row1
	lcd		' '
	movfw	a_ra_0
	Call	lcd10out
	lcd		'h'
	movfw	a_ra_1
	Call	lcd10out
	lcd		'm'
	movfw	a_ra_2
	Call	lcd10out
	lcd		's'
	lcdcontrol	B'11000000'		; home row2
	movlw	'+'
	BTFSC	config,sign_dec		; negativ?
	movlw	'-'
	call	OutLcdDaten
	movfw	a_dec_0
	Call	lcd10out
	lcd		B'11011111'
	movfw	a_dec_1
	Call	lcd10out
	lcd		'\''
	movfw	a_dec_2
	Call	lcd10out
	lcd		'\"'
	return


; #############################################
; #####   Update target position on lcd   #####
; #############################################
UpdateLcdPost
	lcdcontrol	B'10001011'		;home row1
	lcd		' '
	movfw	a_tra_0
	Call	lcd10out
	lcd		'h'
	movfw	a_tra_1
	Call	lcd10out
	lcd		'm'
	movfw	a_tra_2
	Call	lcd10out
	lcd		's'
	lcdcontrol	B'11001011'		; home row2
	movlw	'+'
	BTFSC	config,sign_tdec	; negativ?
	movlw	'-'
	call	OutLcdDaten
	movfw	a_tdec_0
	Call	lcd10out
	lcd		B'11011111'
	movfw	a_tdec_1
	Call	lcd10out
	lcd		'\''
	movfw	a_tdec_2
	Call	lcd10out
	lcd		'\"'
	return


; #########################################################
; #####   Output last 2 decimal digits of w to lcd    #####
; #########################################################
lcd10out
	call	BIN2DEC
	movfw	digit10
	call	OutLcdDaten
	movfw	digit1
	call	OutLcdDaten
	return


; #########################################################
; #####   Output last 2 decimal digits of w to rs232  #####
; #########################################################
RS23210out
	call	BIN2DEC
	movfw	digit10
	call	RS232out
	movfw	digit1
	call	RS232out
	return


; #########################################################
; #####   converts binary numbers to ascii            #####
; #########################################################
BIN2DEC:
		clrf	digit1
		clrf	digit10
		clrf	digit100
		movwf	calctemp
		movlw	D'100'
hundertercounter:
		incf	digit100,	1
		subwf	calctemp,	1
		btfsc   STATUS, 	C	
		goto 	hundertercounter		; carry ?
		decf	digit100,	1
		addwf   calctemp,	1
		movlw	D'10'
zehnercounter:
		incf	digit10,	1	
		subwf	calctemp,	1
		btfsc   STATUS, 	C	
		goto 	zehnercounter			; carry ?
		decf	digit10,	1
		addwf   calctemp,	0
		movwf	digit1					; 
		movlw	.48						; ASCII offset
		addwf	digit1,f
		addwf	digit10,f
		addwf	digit100,f
		return


; #############################################
; #####   Decimal to binary convert       #####
; #############################################
DEC2BIN:
		movlw	D'10'
		movwf	calctemp
		movlw	.208		; ASCII offset
		addwf	digit10,f
		addwf	digit1,w	; save in w
		zehnercounter2:
		addwf	digit10,w
		decfsz  calctemp
		goto 	zehnercounter2			; carry ?
		return


; #############################################
; #####   Wait (1ms * w)                  #####
; #############################################

WAIT
top      movlw   .0           ; timing adjustment variable (1ms)
         movwf   loops2
top2     nop                    ; sit and wait
         nop
         nop
		 nop
		 nop
		 nop
		 nop
		 nop
		 nop
		 nop
		 nop
		 nop
		 nop
         nop
         nop
		 nop
         nop
         decfsz  loops2, F      ; inner loops complete?
         goto    top2           ; no, go again
                                ;
         decfsz  loops, F       ; outer loops complete?
         goto    top            ; no, go again
         retlw   0              ; yes, return from subWAIT


; #############################################
; #####   Output ControlByte to lcd       #####
; #############################################
Control8Bit
		ANDLW	0x0F
		movwf	LcdPort
		bsf		LcdPortE, LcdE
		nop
		bcf		LcdPortE, LcdE
		movlw	D'15'
		movwf	loops
		call 	WAIT
		return


; #############################################
; #####   Wait until lcd is ready         #####
; #############################################
LcdBusy
	    bsf     STATUS, RP0		; make Port A0..3 input
		movlw	B'00001111'
		iorwf   TRISA, f 
	    bcf     STATUS, RP0
		BusyLoop		
		bcf		LcdPort, LcdRs
		bsf		LcdPort, LcdRw	; Lesen
		bsf		LcdPortE, LcdE
		nop
		movf	LcdPort, w
		movwf	LcdStatus
		bcf		LcdPortE, LcdE
		nop
		bsf		LcdPortE, LcdE	; Enable
		nop
		bcf		LcdPortE, LcdE
		btfsc	LcdStatus, 3	; test bit 3
		goto	BusyLoop
		bcf		LcdPort, LcdRw
	    bsf     STATUS, RP0		; make Port A0..3 output
		movlw	B'11110000'
		andwf   TRISA, f    
	    bcf     STATUS, RP0
		nop
		nop
		return	


; #############################################
; #####   Output controlbyte to lcd       #####
; #############################################
OutLcdControl
		movwf	LcdDaten
		swapf	LcdDaten, f			; Hi-part first
		call	LcdBusy
		movf	LcdDaten, w
		andlw	H'0F'
		movwf	LcdPort				; Hi-teil Daten schreiben
		bsf		LcdPortE,  LcdE
		nop
		bcf		LcdPortE,  LcdE		; Disable LcdBus
		swapf	LcdDaten, w
		andlw	H'0F'
		movwf	LcdPort				; Lo-teil Daten schreiben
		bsf		LcdPortE, LcdE
		nop	
		bcf		LcdPortE, LcdE		; Disable LcdBus
		return


; #############################################
; #####   Output databyte to lcd          #####
; #############################################
OutLcdDaten
		movwf	LcdDaten
		swapf	LcdDaten, f
		call	LcdBusy
		movf	LcdDaten, w
		andlw	H'0F'
		movwf	LcdPort			; Hi-teil Daten schreiben
		bsf		LcdPort,  LcdRs	; Daten
		bsf		LcdPortE,  LcdE	; Enable LcdBus
		nop	
		bcf		LcdPortE,  LcdE	; Disable LcdBus	
		swapf	LcdDaten, w
		andlw	H'0F'
		movwf	LcdPort			; Lo-teil Daten schreiben
		bsf		LcdPort, LcdRs	; Daten
		bsf		LcdPortE, LcdE
		nop
		bcf		LcdPortE, LcdE	; Disable LcdBus	
		bcf		LcdPort, LcdRs	;
		return


; #############################################
; #####   Send w via rs232	              #####
; #############################################	
RS232out
    btfss   PIR1,TXIF        ; ist Sender leer ?
    goto    RS232out         ; nein, noch nicht leer
    movwf   TXREG            ; und in den RS232-Sender schreiben 
	return



; #################################################
; #####   several responses needed for lx200  #####
; #################################################
response1:
	out		'+'
	out		'0'
	out		'0'
	out		'#'
	return

response2:
	out		'L'
	out		'X'
	out		'E'
	out		'M'
	out		'U'
	out		'#'
	return

response3:
	out		'0'
	out		'0'
	out		'0'
	out		'*'
	out		'0'
	out		'0'
	out		'`'
	out		'0'
	out		'0'
	out		'#'
	return

response4:
	out		'h'
	out		'o'
	out		'm'
	out		'e'
	out		'#'
	return

response5:
	out		'0'
	out		'0'
	out		':'
	out		'0'
	out		'0'
	out		':'
	out		'0'
	out		'0'
	out		'#'
	return


response6:
	out		'+'
	out		'0'
	out		'0'
	out		'*'
	out		'0'
	out		'0'
	out		'#'
	return

response7:
	out		'-'
	out		'0'
	out		'0'
	out		'0'
	out		'*'
	out		'0'
	out		'0'
	out		'#'
	return



; #############################################
; #####   that's it..                     #####
; #############################################
end

